#!/bin/sh
#

# Source smhi.sh to get facter environment variables
source /etc/profile.d/smhi.sh

# Set max time in seconds to sleep. puppet will randomly sleep from 0 to $splaylimit seconds
# before running
splaylimit=900 # 900s = 15min
#splaylimit=1800 # 1800s = 30min

logfile="/tmp/puppetrun.log"
logfile_old="${logfile}.1"

# Move current logfile to new file to have logs from the two last runs
mv $logfile $logfile_old &>/dev/null

# Set date and time for puppet not to run in respective mode due to security patching
# Repository will be updated in the morning, but we dont want to update all modes at the same time
# Therefore we exit puppet until time reached
factmode=`facter smhi_mode`
factwhere=`facter smhi_where`
day_to_handle_patches="Thursday"
time_utv="10#03"
time_test="10#05"
time_prod="10#12"

date_binary=$(which date 2>/dev/null) 
# Just to be sure, 
if [[ ! $date_binary ]] 
then
  echo "This is weird, cannot find date-binary :(" >> $logfile
  exit 1;
fi

# Check if it's patching day 
if [[ $($date_binary -u +%A) == $day_to_handle_patches ]] 
then
  # Extend splaylimit so we not overload file server
  splaylimit=1800 # 1800s = 30min
  #splaylimit=3000 # 3000s = 50min
 
  if [[ 10#$($date_binary -u +%H) -lt $time_utv && ${factmode:0:3} == "UTV" ]] 
  then 
    echo "This is a utv-server and the time is not ${time_utv:3} yet, we will patch this server later. Now exit puppet" >> $logfile
    exit;
  elif [[ 10#$($date_binary -u +%H) -lt $time_test && $factmode == "TEST" ]] 
  then
    echo "This is a test-server and the time is not ${time_test:3} yet, we will patch this server later. Now exit puppet" >> $logfile
    exit;
  elif [[ 10#$($date_binary -u +%H) -lt $time_prod && $factmode == "PROD" ]] 
  then
    echo "This is a prod-server and the time is not ${time_prod:3} yet, we will patch this server later. Now exit puppet" >> $logfile
    exit;
  fi
fi

# If this is a puppetmaster (i.e listening on port 8300) then we wait to
# execute puppet until the rest of the nodes are done. Reason for this is
# to avoid puppet applied changes (eg autofs restart) affecting other nodes
# which potentially can result in faulty configuration files and changes in 
# rpmlists.
if rpm -q puppet-host-package &>/dev/null;then
  puppetmastersleep=$(($splaylimit+120)) # splaylimit + 2min
  echo "This is a puppetmaster server, delaying puppet run with $puppetmastersleep seconds" >> $logfile
  sleep $puppetmastersleep 
  splaylimit=0 # splay disabled
fi

while (( "$#" ));do
  case "$1" in
    --nosplay)
      splaylimit=0
      # Get output on stdout
      tail -f $logfile & 
      tailpid=$!
      shift
      ;;
    *) 
      echo -e "\nUsage 
      --nosplay - Run puppet directly without any delay."
      shift
    ;;
  esac
done
echo "" > $logfile

# Before doing anything at all, check if puppetdlock-file exists 
# This state-file can be left there if a users turns off the client
# in the middle of a puppetrun. That in turn will lead to that puppet
# wont start again.
if [[ -f /var/lib/puppet/state/agent_catalog_run.lock ]] 
then
  # Lock-file exists but puppet is not running, lets remove the lockfile
  # since it shouldnt be there. 
  pgrep -f "puppet agent" &> /dev/null || rm -rf /var/lib/puppet/state/agent_catalog_run.lock
fi

# Change PWD to make it work via sudo ($HOME will not work due to permissions)
cd /tmp

if ping -c1 puppet.smhi.se &>/dev/null;then
  echo "Starting puppetrun, `date`" >> $logfile
  echo "----------------------" >> $logfile 

  # Run puppet 
  /usr/bin/puppet agent --onetime --verbose --no-daemonize --splay --splaylimit=$splaylimit >> $logfile 2>&1
  puppetretval=$?

  echo "----------------------" >> $logfile 


  # First check for vpn, if true, disable rpm-updates. 
  /sbin/ifconfig | grep vpn0 -q && rpmupdates="false"
  

  # Run correctinstalled.py if puppet run was ok
  if [[ $puppetretval -eq 0 ]];then
    if [[ $rpmupdates != "false" ]]; then

      # If we are a client, make a popup dialog telling the user not to turn off the computer while the updates (if any) are beeing applied. 
      active_user=$(ps aux | egrep '[0-9]+ gnome-session' -m1 | awk '{print $1}')
cat > /tmp/puppetrun-popup.sh <<EOF
  #!/bin/bash

  # In gnome3 (linda4) we must play nice with dbus to get notifications to work, so we have to 
  # export dbus environment variables for the active user in order to get it running.  
  if GNOME_PID=\$(pgrep -u $active_user gnome-session);then
    # Export all environment variables, exclude some weird ones
    eval \$(tr '\0' '\n' < /proc/\${GNOME_PID}/environ | egrep  "DBUS" | sed -e 's/^/export /' -e 's/$/ \&>\/dev\/null/') 
    sleep 10
    if pgrep -f smhi-correctinstalled.py &>/dev/null;then
      notify-send --hint int:transient:1 -u critical "System update in progress" "Please do not shutdown computer until finished (click me to make me disappear)."
     while pgrep -f smhi-correctinstalled.py &>/dev/null;do 
      sleep 2
     done
      notify-send --hint int:transient:1 -u normal "System update done"
    fi
  fi
EOF
      [[ $active_user && -f /tmp/puppetrun-popup.sh ]] && ( su -m $active_user -c "/bin/bash /tmp/puppetrun-popup.sh" & ) &>/dev/null
    
      
      # Special output redirection to get stderr both on screen (to get cron mail) and in 
      # log file. Also with a grep to exclude some annoying errors we dont want to be notified
      # about.
      exclude='Nothing to do|RPMDB altered outside of yum'
      /usr/bin/python /usr/bin/smhi-correctinstalled.py --noremove 2>&1 1>>$logfile | tee -a $logfile | egrep -v "$exclude"
    fi

    [[ -f /tmp/puppetrun-popup.sh ]] && rm -f /tmp/puppetrun-popup.sh

  else
    echo "Puppet run exit code not 0, correctinstalled.py not run." >> $logfile
  fi


fi
echo "----------------------" >> $logfile 
echo "Finished puppetrun, `date`" >> $logfile 
echo "-------------" >> $logfile 

# If we are verbose, kill the tailing. 
[[ $tailpid ]] && kill -9 $tailpid
